<?php
$fileName	= __DIR__ . '/cron.txt';
$pidFile	= __DIR__ . '/cronPid.txt';

define('SERVICE', 'idriveBackup');
define('APPDIR','IDrive');
define('CONFIG','.config');
define('CACHE', '.cache');
define('SCHEDULERJSON', 'scheduler.json');
define('CUSTOMSERVICE', 'immediateCMDviaRoot.php');
define('SYNOLOGY','synology');
define('QNAP','qnap');

define('WEB_PATH_QNAP','/share/Web/');
define('QWEB_PATH_QNAP','/share/Qweb/');
define('PHP_PATH_FOR_QNAP','/mnt/ext/opt/apache/bin/php');
define('PHP_PATH_FOR_SYNOLOGY','/usr/bin/php');
define('WEB_PATH_SNOLOGY','/var/services/');
$DSM7 = 7;

$machine_type = $argv[1];
$webDirPath = "";
$phpPath = "";


if($machine_type == SYNOLOGY){
	$ver = getSynoDSMVersion();
	$wdir = "web" . (($ver >= $DSM7)? '_packages' : '');
	$webDirPath = WEB_PATH_SNOLOGY."$wdir/";
	$phpPath = PHP_PATH_FOR_SYNOLOGY;
	
}
else if($machine_type == QNAP){
	$webDirPath = WEB_PATH_QNAP;
	if(!file_exists($webDirPath))
		$webDirPath = QWEB_PATH_QNAP;
	$phpPath = PHP_PATH_FOR_QNAP;
}
$schedulerContentFile = $webDirPath.SERVICE."/".CACHE."/".SCHEDULERJSON;
$customServiceToRunAsRoot = $webDirPath.APPDIR."/".CONFIG."/".CUSTOMSERVICE;

$schedulerContentArr = array();
$lastModTime = 0;
$needToCheckCutOff = TRUE;
$currDate	= 0; # 01-31
$currWeek	= ""; # Sat
$currHour	= 0; # 00--23
$currMinute	= 0; # 00-59
init();

/*****************************************************************************************************
# Function		: init
# In Param		: 
# Out Param		: 
# Objective		: This function is to start the cron service which is responsible to launch scheduled jobs.
# Added By		: Anil Kumar [25-07-2022]
# Modified By	: Senthil Pandian [06-03-2023]
#****************************************************************************************************/
function init()
{
	GLOBAL $pidFile, $currDate, $currWeek, $currHour, $currMinute, $schedulerContentArr, $schedulerContentFile, $customServiceToRunAsRoot, $lastModTime, $phpPath ;
	GLOBAL $NasUser, $IDriveUser, $backupType, $jobTypes, $jobName;

	if (!isProcessAlive()) {
		file_put_contents($pidFile, getmypid() );
	
		while (true) {
			if(!file_exists($pidFile))
				exit;
			#as json file is not there will be sleeping for a min.
			if(!file_exists($schedulerContentFile))
				sleep(60);
			
			clearstatcache();
			$currTime 	= `date +"%d#%a#%H#%M"`;
			$timeArray 	= explode("#",$currTime);
			$currDate	= (int)$timeArray[0]; # 01-31
			$currWeek	= $timeArray[1]; # Sat
			$currHour	= (int)$timeArray[2]; # 00--23
			$currMinute	= (int)$timeArray[3]; # 00-59
			
// file_put_contents("/tmp/idriveScheduleLog", "\n\n82:currTime:$currTime", FILE_APPEND);//#ToDo: Remove later
			$statData = lstat("$schedulerContentFile");
// file_put_contents("/tmp/idriveScheduleLog", "\n83:statData mtime:".$statData["mtime"].">lastModTime:$lastModTime#\n", FILE_APPEND);//#ToDo: Remove later

			if($statData['mtime'] > $lastModTime){
				$lastModTime = $statData['mtime'];
// file_put_contents("/tmp/idriveScheduleLog", "87:lastModTime:$lastModTime\n", FILE_APPEND);//#ToDo: Remove later

				$schedulerContent = file_get_contents($schedulerContentFile);
				$schedulerContentArr = json_decode($schedulerContent, true);
			}

			foreach($schedulerContentArr as $NasUser => $NasUserContent) {
				foreach($NasUserContent as $IDriveUser => $IDriveUserContent) {
					if(array_key_exists("Status",$IDriveUserContent) && $IDriveUserContent["Status"] == "inactive")
						continue;

					#if user is active we need to parse the jobs to run accordingly.
					foreach($IDriveUserContent as $jobTypes => $jobsInfo) {
						if($jobTypes == "Status")
							continue;
					
						foreach($jobsInfo as $jobName => $jobSettings) {
							if($jobSettings["Scheduler"] == "disabled")
								continue;

							if(($jobTypes == "Backup") || ($jobTypes == "Others"))	
							{
								$backupCmd = getBackupOperationCmd($jobSettings);
// file_put_contents("/tmp/idriveScheduleLog", "114:backupCmd:$backupCmd#\n", FILE_APPEND);//#ToDo: Remove later
								if($backupCmd != "") {
									pclose(popen("{$backupCmd} &","r"));
								}

								$cancleCmd = getCutOffOperationCmd($jobSettings);
// file_put_contents("/tmp/idriveScheduleLog", "120:cancleCmd:$cancleCmd#\n", FILE_APPEND); //#ToDo: Remove later
								if($cancleCmd != "")
									pclose(popen("{$cancleCmd} &","r"));
							}
							else if($jobTypes == "Archive")	
							{	
								$archiveCMD = getArchiveOperationCmd($jobSettings);
								if($archiveCMD != "")
									pclose(popen("{$archiveCMD} &","r"));
							}
						}
					}
				}
			}

			$endTime = time();
			$endMinute = date('i', $endTime); # minute 00-59
			if($currMinute == $endMinute)
			{
				$endSecond = date('s', $endTime); # second 00-59
				$sleepTime = 60 - $endSecond;
				#Post running the existing jobs as per scheduler we are using the remainign time to run the commands which needs to be ran as root
				do{
					$immediateCommand = $phpPath." ". $customServiceToRunAsRoot;
					pclose(popen("{$immediateCommand} &","r"));
					sleep(5);
					$sleepTime -= 5;
				}while($sleepTime>0);
			}
			clearstatcache();
		}
	}
}
/*****************************************************************************************************
# Function		: getBackupOperationCmd
# In Param		: JobSettings
# Out Param		: BackupJobCMD
# Objective		: This function is responsable to parse and find the backup job command if any job needs to launch.
# Added By		: Anil Kumar [25-07-2022]
# Modified By	: Senthil Pandian [06-03-2023]
#****************************************************************************************************/
function getBackupOperationCmd($jobSettings)
{
	GLOBAL $needToCheckCutOff;
	GLOBAL $currDate, $currWeek, $currHour, $currMinute;
	GLOBAL $NasUser, $IDriveUser, $backupType, $jobTypes, $jobName;

	$backupCmd = "";
	$backupType = $jobSettings["Frequency"]["Type"];
	$needToCheckCutOff = TRUE;

	if($backupType == "daily") {
		// $hour = ($jobSettings["Frequency"]["Hour"] == 12)?00:(($jobSettings["Frequency"]["Hour"] == 24)?12:$jobSettings["Frequency"]["Hour"]);
		$hour = $jobSettings["Frequency"]["Hour"];
		if(($currHour == $hour) && ($currMinute == $jobSettings["Frequency"]["Min"])) {
			$backupCmd = $jobSettings["Cmd"]["BackupCmd"];
		}
	}
	else if($backupType == "hourly"){
		if($currMinute == $jobSettings["Frequency"]["Min"]) {
			$backupCmd = $jobSettings["Cmd"]["BackupCmd"];
		}
		#no cutoff for hourly
		$needToCheckCutOff = FALSE;
	}
	else if(($backupType == "monthly")){
		// $hour = ($jobSettings["Frequency"]["Hour"] == 12)?00:(($jobSettings["Frequency"]["Hour"] == 24)?12:$jobSettings["Frequency"]["Hour"]);
		$hour = $jobSettings["Frequency"]["Hour"];
		if(($currDate == $jobSettings["Frequency"]["DayOfMonth"]) && ($currHour == $hour) && ($currMinute == $jobSettings["Frequency"]["Min"])) {
			$backupCmd = $jobSettings["Cmd"]["BackupCmd"];
		}

		if($jobSettings["Frequency"]["DayOfMonth"] < $currDate)
			unset($jobsInfo[$jobName]);
	}
	else if(($backupType == "weekly") && (in_array(strtoupper($currWeek),$jobSettings["Frequency"]["Days"]) || (in_array(strtoupper($currWeek), explode(",", $jobSettings["Frequency"]["Days"][0]))))){
		// $hour = ($jobSettings["Frequency"]["Hour"] == 12)?00:(($jobSettings["Frequency"]["Hour"] == 24)?12:$jobSettings["Frequency"]["Hour"]);
		$hour = $jobSettings["Frequency"]["Hour"];
		if(($currHour == $hour) && ($currMinute == $jobSettings["Frequency"]["Min"])) {
			$backupCmd = $jobSettings["Cmd"]["BackupCmd"];
		}
	}
	else if(($backupType == "startImmediately")){
		// $hour = ($currHour == 12)?00:(($currHour == 24)?12:($currHour>12?($currHour-12):$currHour));
		$hour =(($currHour == 24)?12:($currHour>12?($currHour-12):$currHour));
		$hour = sprintf("%02d", $hour);
		$currMinute = sprintf("%02d", $currMinute);
		$schTimeMer = ($currHour < 12 || $currHour == 24)?"AM":"PM";
		$schTime = $hour .":".$currMinute." ".$schTimeMer;
		$schTime = strtotime("today $schTime");
// file_put_contents("/tmp/idriveScheduleLog", "199:schTime:$schTime#jobSettings Date:".$jobSettings["Frequency"]["Date"]."#jobTypes:$jobTypes\n\n", FILE_APPEND); //#ToDo: Remove later
		if($jobSettings["Frequency"]["Date"] == $schTime) {
			if($jobTypes == "Others"){
				$backupCmd = $jobSettings["Cmd"];
				deleteSchedulerInfo($NasUser, $IDriveUser, $jobTypes, $jobName);
			}
			else {
				$backupCmd = $jobSettings["Cmd"]["BackupCmd"];
			}
		}
// file_put_contents("/tmp/idriveScheduleLog", "210:backupType:$backupType#schTime:$schTime#jobSettings Date:".$jobSettings["Frequency"]["Date"]."#jobTypes:$jobTypes\n\n", FILE_APPEND); //#ToDo: Remove later
	}
	return $backupCmd;
}
/*****************************************************************************************************
# Function		: getCutOffOperationCmd
# In Param		: JobSettings
# Out Param		: JobTerminationCMD
# Objective		: This function is to get the jobTerminationCMD if anything is suitable for the moment.
# Added By		: Anil Kumar [25-07-2022]
# Modified By	: <name> [Date]
#****************************************************************************************************/
function getCutOffOperationCmd($jobSettings)
{
	GLOBAL $needToCheckCutOff;
	GLOBAL  $currHour, $currMinute;
	$cancleCmd = "";
	$cutOffStatus = $jobSettings["CuttOff"]["Status"];
	
	if((!$needToCheckCutOff) || ($cutOffStatus == "disabled"))
		return 0;
	
	$cutOffHour = ($jobSettings["CuttOff"]["Frequency"]["Hour"] == 12)?00:(($jobSettings["CuttOff"]["Frequency"]["Hour"] == 24)?12:$jobSettings["CuttOff"]["Frequency"]["Hour"]);
	if(($currHour == $cutOffHour) && ($currMinute == $jobSettings["CuttOff"]["Frequency"]["Min"])){
		$cancleCmd = $jobSettings["Cmd"]["CancleCmd"];
	}
	return $cancleCmd;
}

/*****************************************************************************************************
# Function		: getArchiveOperationCmd
# In Param		: JobSettings
# Out Param		: ArchiveJobCommand
# Objective		: This function is to get the Archive job .
# Added By		: Anil Kumar [25-07-2022]
# Modified By	: <name> [Date]
#****************************************************************************************************/
function getArchiveOperationCmd($jobSettings)
{
	$archiveCMD = "";
	GLOBAL $currDate, $currWeek, $currHour, $currMinute;
	#Archive Job
	$archiveDaysFre = $jobSettings["Frequency"]["NoOfDays"];
	$archivePerFre = $jobSettings["Frequency"]["Per"];
	$archiveHour = $jobSettings["Frequency"]["Hour"];
	$archiveMin = $jobSettings["Frequency"]["Min"];
								
	if(($currHour == $jobSettings["Frequency"]["Hour"]) && ($currMinute == $jobSettings["Frequency"]["Min"])) {
		$archiveCMD = $jobSettings["Cmd"];
	}
	return $archiveCMD;
}
/*********************************************************************************************************
* Subroutine Name       : getSynoDSMVersion
* Objective             : Finds the Synology DSM version
* Added By              : Sabin Cheruvattil
**********************************************************************************************************/
function getSynoDSMVersion() {
	$verarr = explode("\n", `cat /etc.defaults/VERSION`);
	array_walk($verarr, 'trim');

	$kvp = preg_split('/=/', $verarr[0]);
	$ver = (int)trim($kvp[1], '"');

	return $ver;
}
/*****************************************************************************************************
# Function		: isProcessAlive
# In Param		: 
# Out Param		: TRUE/FALSE which denotes process is running or not
# Objective		: This function is responsible to check is cron process really running
# Added By		: Anil Kumar [25-07-2022]
# Modified By	: <name> [Date]
#****************************************************************************************************/
function isProcessAlive() {
	
	GLOBAL $pidFile;
	$pid = 0;
	if(file_exists($pidFile))
		$pid = file_get_contents($pidFile);
	if($pid == 0)
		return FALSE;
	
	$psPath = getPsPath();
	$psCmd = $psPath." | grep ".$pid;
	$pidList = `$psCmd`;
	if($pidList != "" && (strpos($pidList, "IDrive_cron") !== false)){
		return TRUE;
	}
	return FALSE;
}
/*****************************************************************************************************
# Function		: getPsPath
# In Param		: 
# Out Param		: PS path based on machine
# Objective		: This function is responsible desciding what ps command need to be used based on machine
# Added By		: Anil Kumar [25-07-2022]
# Modified By	: <name> [Date]
#****************************************************************************************************/
function getPsPath() {
	GLOBAL $machine_type; 
	$psPath ="";
	if("synology" == $machine_type){
		$ver = getSynoDSMVersion();
		if($ver >= 6)
			$psPath = "ps -elf";
		else
			$psPath = "ps -wwwww";
	}
	else if("qnap" == $machine_type){
		$psPath = "ps";
	}
	else if(("netgear" == $machine_type) ||("Asustor" == $machine_type)){
		$psPath = "ps -elf";
	}
	return $psPath;
}
/*****************************************************************************************************
# Function		: deleteSchedulerInfo
# In Param		: NAS user, IDrive user, Job type, Job name
# Out Param		: 
# Objective		: This function is responsible for deleting the desired job from scheduler info.
# Added By		: Anil Kumar [25-07-2022]
# Modified By	: <name> [Date]
#****************************************************************************************************/
function deleteSchedulerInfo($nasUserName, $idriveUserName, $jobType, $job)
{
	GLOBAL $schedulerContentFile, $schedulerContentArr;
	$schedulerContent = file_get_contents($schedulerContentFile);
	$schedulerContentArr = json_decode($schedulerContent, true);
	unset($schedulerContentArr[$nasUserName][$idriveUserName][$jobType][$job]);
	file_put_contents($schedulerContentFile, json_encode($schedulerContentArr,TRUE));
	return 1;
}



?>