#!/bin/sh

#-------------------------------------------------------------------------------
# IDrive su Binary Checker
#
# Version: 1.0
# Created By : Sabin Cheruvattil @ IDrive Inc
# Reviewed By: Deepak Chaurasia @ IDrive Inc
#-------------------------------------------------------------------------------

# Optional Parameters: 
# --path : To which directory EVS binaries get copied, source binary zip path
# --machine: Type of the machine we are runinning the test. [qnap]
# Usage:
# ./subinaryChk.sh
# ./subinaryChk.sh --path="/tmp/" --machine=qnap

while [ "$#" -gt 0 ]; do
	case "$1" in
		--machine|--path|--machine=|--path=) echo "$1 requires an argument" >&2; exit 1;;
		--machine=*) machineType="${1#*=}"; shift 1;;
		--path=*) binaryPath="${1#*=}"; shift 1;;

		-*) echo "Unknown option: $1" >&2; exit 1;;
		*) junk="$1"; shift 1;;
	esac
done

if [ "${binaryPath}" = "" ]; then
	echo "--path is required";
	exit;
fi

# As of now we are using this script for qnap platforms. When we have more su binaries we can remove the following line
if [ "${machineType}" = "" ]; then
	machineType="qnap";
fi

SuQnapBin32=$binaryPath"/IdriveSu/qnap_x86.zip";
SuQnapBin64=$binaryPath"/IdriveSu/qnap_x86_64.zip";
SuQnapArmBin=$binaryPath"/IdriveSu/qnap_arm_x09.zip";
SuQnapAarchBin=$binaryPath"/IdriveSu/qnap_arm_64.zip";

currentDir=$PWD;
tracelog="$currentDir/trace.log";
MACHINE_ARCH=$(uname -m);
MACHINE_ARCH_DATA=$(uname -a);

if [ "${machineType}" == "qnap" ]; then
	if [ `echo "${MACHINE_ARCH}" | grep -i 'armv'` ]; then
		arrayLinks="$SuQnapArmBin $SuQnapAarchBin";
	elif [ "${MACHINE_ARCH}" = 'x86' ] || [ "${MACHINE_ARCH}" = 'i686' ] || [ "${MACHINE_ARCH}" = 'i386' ] || [ "${MACHINE_ARCH}" = 'i586' ]; then
		arrayLinks="$SuQnapBin32 $SuQnapBin64";
	elif [ "${MACHINE_ARCH}" == 'x86_64' ] || [ "${MACHINE_ARCH}" = 'amd64' ]; then
		arrayLinks="$SuQnapBin64 $SuQnapBin32";
	elif [ "${MACHINE_ARCH}" == 'aarch64' ]; then
		arrayLinks="$SuQnapAarchBin $SuQnapArmBin"
	fi
fi

if [ "${arrayLinks}" = '' ]; then
	arrayLinks="$SuQnapBin64 $SuQnapBin32 $SuQnapArmBin $SuQnapAarchBin";
fi

# Get zip/unzip type
sevenUnzip=`which 7z 2>/dev/null`;
if [ $sevenUnzip != '' ]; then
	unzip="$sevenUnzip x";
else
	unzip=`which unzip 2>/dev/null`;
fi

if [ "${unzip}" = "" ]; then
	echo "No unzip available";
	echo "No unzip available" >> $tracelog;
	exit;
fi  

workStat=0;
for i in $arrayLinks; do
	zipName=`echo "${i##*/}"`;
	cp -rfp $i $currentDir;
	$unzip "$zipName";

	zipFullPath=`echo "$zipName" | cut -d. -f1`;
	suDir="$currentDir/$zipFullPath";
	su="$suDir/su";
	chmod 04755 "$su";

	`$su --help 1>out.txt 2>>out.txt`;
	suHelpResLen=`wc -l out.txt | awk -F " " '{print $1}'`;

	rm "$zipName";
	rm "out.txt";

	if [ "${suHelpResLen}" -gt "5" ]; then
		cp -rfp $su "$binaryPath/";
		workStat=1;
	fi

	if [ "$suDir" != "/" ]; then
		rm -rf "$suDir";
	fi

	if [ "$workStat" -eq "1" ]; then
		break;
	fi
done

echo "";
echo "";
echo "Send the below details to support:";
echo "------------------------------------";

if [ "${machineType}" != "" ]; then
	echo "Machine Details: ${machineType}";
fi

echo "OS Details: ${MACHINE_ARCH_DATA}";
if [ $workStat -eq 1 ]; then
	echo "Working zip is $zipName";
elif [ $workStat -eq 0 ]; then
	echo "Unable to find compatible su binary";
fi
